#include <M5Stack.h>
#include <WiFi.h>
#include <WiFiClientSecure.h>
#include <PubSubClient.h>
#include "Free_Fonts.h"
#include "secret.h"

#define PIR_MOTION_SENSOR 36
#define CLIENT_ID "1"

static WiFiClientSecure httpsClient;
static PubSubClient mqttClient(httpsClient);
static int detected = 0;

// WiFi接続する
void setupWifi() {
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED) {
    M5.Lcd.print(".");
    delay(1000);
  }
}

// MQTT接続する
void setupMqtt() {
  httpsClient.setCACert(ROOT_CA);
  httpsClient.setCertificate(CERTIFICATE);
  httpsClient.setPrivateKey(PRIVATE_KEY);
  mqttClient.setServer(CLOUD_ENDPOINT, CLOUD_PORT);
  while (!mqttClient.connected()) {
    if (!(mqttClient.connect(CLIENT_ID))) {
      delay(1000);
    }
  }
}

// LCDを初期化する
void setupLcd() {
  M5.Lcd.fillScreen(TFT_BLACK);
  M5.Lcd.setTextDatum(MC_DATUM);
  M5.Lcd.setFreeFont(FF44);
}

// 検知状態を送信する
void sendDetected(int detected) {
  // 検知状態をJSON化する
  char message[64] = {0};
  sprintf(message, "{\"detected\": %d}", detected);

  // JSONを送信する
  while(!(mqttClient.publish(CLOUD_TOPIC, message))) {
    // 送信失敗した場合は再接続してリトライする
    mqttClient.disconnect();
    while (!mqttClient.connected()) {
      if (!(mqttClient.connect(CLIENT_ID))) {
        M5.Lcd.print("#");
        delay(1000);
      }
    }
  }
}

void setup() {
  M5.begin();
  M5.Power.begin();
  // setupWifi(); TODO: Secret.hを編集する必要がある
  // setupMqtt(); TODO: Secret.hを編集する必要がある
  setupLcd();
  pinMode(PIR_MOTION_SENSOR, INPUT);
}

void loop() {
  // 検知状態を読み出し
  int ret = digitalRead(PIR_MOTION_SENSOR);
  if (ret != detected) {
    // 検知状態が変化した場合
    detected = ret;
    if (detected) {
      // 検知したとき: Hello!と表示しクラウドへ送信
      M5.Lcd.fillScreen(TFT_BLUE);
      M5.Lcd.setTextColor(TFT_WHITE, TFT_BLUE);
      M5.Lcd.drawString("Hello!", 160, 120, GFXFF);
    } else {
      // 検知していないとき
      M5.Lcd.fillScreen(TFT_BLACK);
      M5.Lcd.setTextColor(TFT_WHITE, TFT_BLACK);
      M5.Lcd.drawString("....", 160, 120, GFXFF);
    }
    // sendDetected(detected); TODO: Secret.hを編集する必要がある
    delay(1000);
  }
}
